#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ActionWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "id")]
    pub id: Option<String>,
    #[serde(rename = "label")]
    pub label: String,
}
#[derive(Debug, Encode, Decode)]
pub enum ActionPanelSectionWidgetOrderedMembers {
    Action(ActionWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct ActionPanelSectionWidgetContent {
    pub ordered_members: Vec<ActionPanelSectionWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for ActionPanelSectionWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum ActionPanelSectionWidgetMembersOwned {
            #[serde(rename = "gauntlet:action")]
            Action(ActionWidget),
        }
        let mut members = Vec::<ActionPanelSectionWidgetMembersOwned>::deserialize(deserializer)?;

        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                ActionPanelSectionWidgetMembersOwned::Action(widget) => {
                    ordered_members.insert(0, ActionPanelSectionWidgetOrderedMembers::Action(widget));
                }
            }
        }

        Ok(ActionPanelSectionWidgetContent {
            ordered_members
        })
    }
}
impl Serialize for ActionPanelSectionWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum ActionPanelSectionWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:action")]
            Action(&'a ActionWidget),
        }
        let mut members = Vec::<ActionPanelSectionWidgetMembersRef>::new();

        for member in &self.ordered_members {
            match member {
                ActionPanelSectionWidgetOrderedMembers::Action(widget) => {
                    members.push(ActionPanelSectionWidgetMembersRef::Action(widget))
                }
            }
        }

        Vec::<ActionPanelSectionWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ActionPanelSectionWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "title")]
    pub title: Option<String>,
    pub content: ActionPanelSectionWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub enum ActionPanelWidgetOrderedMembers {
    Action(ActionWidget),
    ActionPanelSection(ActionPanelSectionWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct ActionPanelWidgetContent {
    pub ordered_members: Vec<ActionPanelWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for ActionPanelWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum ActionPanelWidgetMembersOwned {
            #[serde(rename = "gauntlet:action")]
            Action(ActionWidget),
            #[serde(rename = "gauntlet:action_panel_section")]
            ActionPanelSection(ActionPanelSectionWidget),
        }
        let mut members = Vec::<ActionPanelWidgetMembersOwned>::deserialize(deserializer)?;

        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                ActionPanelWidgetMembersOwned::Action(widget) => {
                    ordered_members.insert(0, ActionPanelWidgetOrderedMembers::Action(widget));
                }
                ActionPanelWidgetMembersOwned::ActionPanelSection(widget) => {
                    ordered_members.insert(0, ActionPanelWidgetOrderedMembers::ActionPanelSection(widget));
                }
            }
        }

        Ok(ActionPanelWidgetContent {
            ordered_members
        })
    }
}
impl Serialize for ActionPanelWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum ActionPanelWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:action")]
            Action(&'a ActionWidget),
            #[serde(rename = "gauntlet:action_panel_section")]
            ActionPanelSection(&'a ActionPanelSectionWidget),
        }
        let mut members = Vec::<ActionPanelWidgetMembersRef>::new();

        for member in &self.ordered_members {
            match member {
                ActionPanelWidgetOrderedMembers::Action(widget) => {
                    members.push(ActionPanelWidgetMembersRef::Action(widget))
                }
                ActionPanelWidgetOrderedMembers::ActionPanelSection(widget) => {
                    members.push(ActionPanelWidgetMembersRef::ActionPanelSection(widget))
                }
            }
        }

        Vec::<ActionPanelWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ActionPanelWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "title")]
    pub title: Option<String>,
    pub content: ActionPanelWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct MetadataLinkWidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for MetadataLinkWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum MetadataLinkWidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<MetadataLinkWidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                MetadataLinkWidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(MetadataLinkWidgetContent {
            text
        })
    }
}
impl Serialize for MetadataLinkWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum MetadataLinkWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<MetadataLinkWidgetMembersRef>::new();

        for value in &self.text {
            members.push(MetadataLinkWidgetMembersRef::Text { value });
        }

        Vec::<MetadataLinkWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct MetadataLinkWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "label")]
    pub label: String,
    #[serde(rename = "href")]
    pub href: String,
    pub content: MetadataLinkWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct MetadataTagItemWidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for MetadataTagItemWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum MetadataTagItemWidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<MetadataTagItemWidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                MetadataTagItemWidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(MetadataTagItemWidgetContent {
            text
        })
    }
}
impl Serialize for MetadataTagItemWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum MetadataTagItemWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<MetadataTagItemWidgetMembersRef>::new();

        for value in &self.text {
            members.push(MetadataTagItemWidgetMembersRef::Text { value });
        }

        Vec::<MetadataTagItemWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct MetadataTagItemWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: MetadataTagItemWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub enum MetadataTagListWidgetOrderedMembers {
    MetadataTagItem(MetadataTagItemWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct MetadataTagListWidgetContent {
    pub ordered_members: Vec<MetadataTagListWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for MetadataTagListWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum MetadataTagListWidgetMembersOwned {
            #[serde(rename = "gauntlet:metadata_tag_item")]
            MetadataTagItem(MetadataTagItemWidget),
        }
        let mut members = Vec::<MetadataTagListWidgetMembersOwned>::deserialize(deserializer)?;

        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                MetadataTagListWidgetMembersOwned::MetadataTagItem(widget) => {
                    ordered_members.insert(0, MetadataTagListWidgetOrderedMembers::MetadataTagItem(widget));
                }
            }
        }

        Ok(MetadataTagListWidgetContent {
            ordered_members
        })
    }
}
impl Serialize for MetadataTagListWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum MetadataTagListWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:metadata_tag_item")]
            MetadataTagItem(&'a MetadataTagItemWidget),
        }
        let mut members = Vec::<MetadataTagListWidgetMembersRef>::new();

        for member in &self.ordered_members {
            match member {
                MetadataTagListWidgetOrderedMembers::MetadataTagItem(widget) => {
                    members.push(MetadataTagListWidgetMembersRef::MetadataTagItem(widget))
                }
            }
        }

        Vec::<MetadataTagListWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct MetadataTagListWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "label")]
    pub label: String,
    pub content: MetadataTagListWidgetContent,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct MetadataSeparatorWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
}
#[derive(Debug, Encode, Decode)]
pub struct MetadataValueWidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for MetadataValueWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum MetadataValueWidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<MetadataValueWidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                MetadataValueWidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(MetadataValueWidgetContent {
            text
        })
    }
}
impl Serialize for MetadataValueWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum MetadataValueWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<MetadataValueWidgetMembersRef>::new();

        for value in &self.text {
            members.push(MetadataValueWidgetMembersRef::Text { value });
        }

        Vec::<MetadataValueWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct MetadataValueWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "label")]
    pub label: String,
    pub content: MetadataValueWidgetContent,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct MetadataIconWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "icon")]
    pub icon: Icons,
    #[serde(rename = "label")]
    pub label: String,
}
#[derive(Debug, Encode, Decode)]
pub enum MetadataWidgetOrderedMembers {
    MetadataTagList(MetadataTagListWidget),
    MetadataLink(MetadataLinkWidget),
    MetadataValue(MetadataValueWidget),
    MetadataIcon(MetadataIconWidget),
    MetadataSeparator(MetadataSeparatorWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct MetadataWidgetContent {
    pub ordered_members: Vec<MetadataWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for MetadataWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum MetadataWidgetMembersOwned {
            #[serde(rename = "gauntlet:metadata_tag_list")]
            MetadataTagList(MetadataTagListWidget),
            #[serde(rename = "gauntlet:metadata_link")]
            MetadataLink(MetadataLinkWidget),
            #[serde(rename = "gauntlet:metadata_value")]
            MetadataValue(MetadataValueWidget),
            #[serde(rename = "gauntlet:metadata_icon")]
            MetadataIcon(MetadataIconWidget),
            #[serde(rename = "gauntlet:metadata_separator")]
            MetadataSeparator(MetadataSeparatorWidget),
        }
        let mut members = Vec::<MetadataWidgetMembersOwned>::deserialize(deserializer)?;

        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                MetadataWidgetMembersOwned::MetadataTagList(widget) => {
                    ordered_members.insert(0, MetadataWidgetOrderedMembers::MetadataTagList(widget));
                }
                MetadataWidgetMembersOwned::MetadataLink(widget) => {
                    ordered_members.insert(0, MetadataWidgetOrderedMembers::MetadataLink(widget));
                }
                MetadataWidgetMembersOwned::MetadataValue(widget) => {
                    ordered_members.insert(0, MetadataWidgetOrderedMembers::MetadataValue(widget));
                }
                MetadataWidgetMembersOwned::MetadataIcon(widget) => {
                    ordered_members.insert(0, MetadataWidgetOrderedMembers::MetadataIcon(widget));
                }
                MetadataWidgetMembersOwned::MetadataSeparator(widget) => {
                    ordered_members.insert(0, MetadataWidgetOrderedMembers::MetadataSeparator(widget));
                }
            }
        }

        Ok(MetadataWidgetContent {
            ordered_members
        })
    }
}
impl Serialize for MetadataWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum MetadataWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:metadata_tag_list")]
            MetadataTagList(&'a MetadataTagListWidget),
            #[serde(rename = "gauntlet:metadata_link")]
            MetadataLink(&'a MetadataLinkWidget),
            #[serde(rename = "gauntlet:metadata_value")]
            MetadataValue(&'a MetadataValueWidget),
            #[serde(rename = "gauntlet:metadata_icon")]
            MetadataIcon(&'a MetadataIconWidget),
            #[serde(rename = "gauntlet:metadata_separator")]
            MetadataSeparator(&'a MetadataSeparatorWidget),
        }
        let mut members = Vec::<MetadataWidgetMembersRef>::new();

        for member in &self.ordered_members {
            match member {
                MetadataWidgetOrderedMembers::MetadataTagList(widget) => {
                    members.push(MetadataWidgetMembersRef::MetadataTagList(widget))
                }
                MetadataWidgetOrderedMembers::MetadataLink(widget) => {
                    members.push(MetadataWidgetMembersRef::MetadataLink(widget))
                }
                MetadataWidgetOrderedMembers::MetadataValue(widget) => {
                    members.push(MetadataWidgetMembersRef::MetadataValue(widget))
                }
                MetadataWidgetOrderedMembers::MetadataIcon(widget) => {
                    members.push(MetadataWidgetMembersRef::MetadataIcon(widget))
                }
                MetadataWidgetOrderedMembers::MetadataSeparator(widget) => {
                    members.push(MetadataWidgetMembersRef::MetadataSeparator(widget))
                }
            }
        }

        Vec::<MetadataWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct MetadataWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: MetadataWidgetContent,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ImageWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "source")]
    pub source: ImageLike,
}
#[derive(Debug, Encode, Decode)]
pub struct H1WidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for H1WidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum H1WidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<H1WidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                H1WidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(H1WidgetContent {
            text
        })
    }
}
impl Serialize for H1WidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum H1WidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<H1WidgetMembersRef>::new();

        for value in &self.text {
            members.push(H1WidgetMembersRef::Text { value });
        }

        Vec::<H1WidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct H1Widget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: H1WidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct H2WidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for H2WidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum H2WidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<H2WidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                H2WidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(H2WidgetContent {
            text
        })
    }
}
impl Serialize for H2WidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum H2WidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<H2WidgetMembersRef>::new();

        for value in &self.text {
            members.push(H2WidgetMembersRef::Text { value });
        }

        Vec::<H2WidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct H2Widget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: H2WidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct H3WidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for H3WidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum H3WidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<H3WidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                H3WidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(H3WidgetContent {
            text
        })
    }
}
impl Serialize for H3WidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum H3WidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<H3WidgetMembersRef>::new();

        for value in &self.text {
            members.push(H3WidgetMembersRef::Text { value });
        }

        Vec::<H3WidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct H3Widget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: H3WidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct H4WidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for H4WidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum H4WidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<H4WidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                H4WidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(H4WidgetContent {
            text
        })
    }
}
impl Serialize for H4WidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum H4WidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<H4WidgetMembersRef>::new();

        for value in &self.text {
            members.push(H4WidgetMembersRef::Text { value });
        }

        Vec::<H4WidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct H4Widget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: H4WidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct H5WidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for H5WidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum H5WidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<H5WidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                H5WidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(H5WidgetContent {
            text
        })
    }
}
impl Serialize for H5WidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum H5WidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<H5WidgetMembersRef>::new();

        for value in &self.text {
            members.push(H5WidgetMembersRef::Text { value });
        }

        Vec::<H5WidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct H5Widget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: H5WidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct H6WidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for H6WidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum H6WidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<H6WidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                H6WidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(H6WidgetContent {
            text
        })
    }
}
impl Serialize for H6WidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum H6WidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<H6WidgetMembersRef>::new();

        for value in &self.text {
            members.push(H6WidgetMembersRef::Text { value });
        }

        Vec::<H6WidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct H6Widget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: H6WidgetContent,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct HorizontalBreakWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
}
#[derive(Debug, Encode, Decode)]
pub struct CodeBlockWidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for CodeBlockWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum CodeBlockWidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<CodeBlockWidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                CodeBlockWidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(CodeBlockWidgetContent {
            text
        })
    }
}
impl Serialize for CodeBlockWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum CodeBlockWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<CodeBlockWidgetMembersRef>::new();

        for value in &self.text {
            members.push(CodeBlockWidgetMembersRef::Text { value });
        }

        Vec::<CodeBlockWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct CodeBlockWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: CodeBlockWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct ParagraphWidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for ParagraphWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum ParagraphWidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<ParagraphWidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                ParagraphWidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(ParagraphWidgetContent {
            text
        })
    }
}
impl Serialize for ParagraphWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum ParagraphWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<ParagraphWidgetMembersRef>::new();

        for value in &self.text {
            members.push(ParagraphWidgetMembersRef::Text { value });
        }

        Vec::<ParagraphWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ParagraphWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: ParagraphWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub enum ContentWidgetOrderedMembers {
    Paragraph(ParagraphWidget),
    Image(ImageWidget),
    H1(H1Widget),
    H2(H2Widget),
    H3(H3Widget),
    H4(H4Widget),
    H5(H5Widget),
    H6(H6Widget),
    HorizontalBreak(HorizontalBreakWidget),
    CodeBlock(CodeBlockWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct ContentWidgetContent {
    pub ordered_members: Vec<ContentWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for ContentWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum ContentWidgetMembersOwned {
            #[serde(rename = "gauntlet:paragraph")]
            Paragraph(ParagraphWidget),
            #[serde(rename = "gauntlet:image")]
            Image(ImageWidget),
            #[serde(rename = "gauntlet:h1")]
            H1(H1Widget),
            #[serde(rename = "gauntlet:h2")]
            H2(H2Widget),
            #[serde(rename = "gauntlet:h3")]
            H3(H3Widget),
            #[serde(rename = "gauntlet:h4")]
            H4(H4Widget),
            #[serde(rename = "gauntlet:h5")]
            H5(H5Widget),
            #[serde(rename = "gauntlet:h6")]
            H6(H6Widget),
            #[serde(rename = "gauntlet:horizontal_break")]
            HorizontalBreak(HorizontalBreakWidget),
            #[serde(rename = "gauntlet:code_block")]
            CodeBlock(CodeBlockWidget),
        }
        let mut members = Vec::<ContentWidgetMembersOwned>::deserialize(deserializer)?;

        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                ContentWidgetMembersOwned::Paragraph(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::Paragraph(widget));
                }
                ContentWidgetMembersOwned::Image(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::Image(widget));
                }
                ContentWidgetMembersOwned::H1(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::H1(widget));
                }
                ContentWidgetMembersOwned::H2(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::H2(widget));
                }
                ContentWidgetMembersOwned::H3(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::H3(widget));
                }
                ContentWidgetMembersOwned::H4(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::H4(widget));
                }
                ContentWidgetMembersOwned::H5(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::H5(widget));
                }
                ContentWidgetMembersOwned::H6(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::H6(widget));
                }
                ContentWidgetMembersOwned::HorizontalBreak(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::HorizontalBreak(widget));
                }
                ContentWidgetMembersOwned::CodeBlock(widget) => {
                    ordered_members.insert(0, ContentWidgetOrderedMembers::CodeBlock(widget));
                }
            }
        }

        Ok(ContentWidgetContent {
            ordered_members
        })
    }
}
impl Serialize for ContentWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum ContentWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:paragraph")]
            Paragraph(&'a ParagraphWidget),
            #[serde(rename = "gauntlet:image")]
            Image(&'a ImageWidget),
            #[serde(rename = "gauntlet:h1")]
            H1(&'a H1Widget),
            #[serde(rename = "gauntlet:h2")]
            H2(&'a H2Widget),
            #[serde(rename = "gauntlet:h3")]
            H3(&'a H3Widget),
            #[serde(rename = "gauntlet:h4")]
            H4(&'a H4Widget),
            #[serde(rename = "gauntlet:h5")]
            H5(&'a H5Widget),
            #[serde(rename = "gauntlet:h6")]
            H6(&'a H6Widget),
            #[serde(rename = "gauntlet:horizontal_break")]
            HorizontalBreak(&'a HorizontalBreakWidget),
            #[serde(rename = "gauntlet:code_block")]
            CodeBlock(&'a CodeBlockWidget),
        }
        let mut members = Vec::<ContentWidgetMembersRef>::new();

        for member in &self.ordered_members {
            match member {
                ContentWidgetOrderedMembers::Paragraph(widget) => {
                    members.push(ContentWidgetMembersRef::Paragraph(widget))
                }
                ContentWidgetOrderedMembers::Image(widget) => {
                    members.push(ContentWidgetMembersRef::Image(widget))
                }
                ContentWidgetOrderedMembers::H1(widget) => {
                    members.push(ContentWidgetMembersRef::H1(widget))
                }
                ContentWidgetOrderedMembers::H2(widget) => {
                    members.push(ContentWidgetMembersRef::H2(widget))
                }
                ContentWidgetOrderedMembers::H3(widget) => {
                    members.push(ContentWidgetMembersRef::H3(widget))
                }
                ContentWidgetOrderedMembers::H4(widget) => {
                    members.push(ContentWidgetMembersRef::H4(widget))
                }
                ContentWidgetOrderedMembers::H5(widget) => {
                    members.push(ContentWidgetMembersRef::H5(widget))
                }
                ContentWidgetOrderedMembers::H6(widget) => {
                    members.push(ContentWidgetMembersRef::H6(widget))
                }
                ContentWidgetOrderedMembers::HorizontalBreak(widget) => {
                    members.push(ContentWidgetMembersRef::HorizontalBreak(widget))
                }
                ContentWidgetOrderedMembers::CodeBlock(widget) => {
                    members.push(ContentWidgetMembersRef::CodeBlock(widget))
                }
            }
        }

        Vec::<ContentWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ContentWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: ContentWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct DetailWidgetContent {
    pub actions: Option<ActionPanelWidget>,
    pub metadata: Option<MetadataWidget>,
    pub content: Option<ContentWidget>,
}
impl<'de> Deserialize<'de> for DetailWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum DetailWidgetMembersOwned {
            #[serde(rename = "gauntlet:metadata")]
            Metadata(MetadataWidget),
            #[serde(rename = "gauntlet:content")]
            Content(ContentWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(ActionPanelWidget),
        }
        let mut members = Vec::<DetailWidgetMembersOwned>::deserialize(deserializer)?;

        let mut actions: Option<_> = None;
        let mut metadata: Option<_> = None;
        let mut content: Option<_> = None;

        while let Some(member) = members.pop() {
            match member {
                DetailWidgetMembersOwned::ActionPanel(widget) => {
                    if let Some(_) = actions {
                        return Err(Error::custom("Only one ActionPanel is allowed"))
                    }
                    actions = Some(widget);
                }
                DetailWidgetMembersOwned::Metadata(widget) => {
                    if let Some(_) = metadata {
                        return Err(Error::custom("Only one Metadata is allowed"))
                    }
                    metadata = Some(widget);
                }
                DetailWidgetMembersOwned::Content(widget) => {
                    if let Some(_) = content {
                        return Err(Error::custom("Only one Content is allowed"))
                    }
                    content = Some(widget);
                }
            }
        }

        Ok(DetailWidgetContent {
            metadata,
            content,
            actions,
        })
    }
}
impl Serialize for DetailWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum DetailWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:metadata")]
            Metadata(&'a MetadataWidget),
            #[serde(rename = "gauntlet:content")]
            Content(&'a ContentWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(&'a ActionPanelWidget),
        }
        let mut members = Vec::<DetailWidgetMembersRef>::new();

        if let Some(actions) = &self.actions {
            members.push(DetailWidgetMembersRef::ActionPanel(actions))
        }
        if let Some(item) = &self.metadata {
            members.push(DetailWidgetMembersRef::Metadata(item))
        }
        if let Some(item) = &self.content {
            members.push(DetailWidgetMembersRef::Content(item))
        }

        Vec::<DetailWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct DetailWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "isLoading")]
    pub is_loading: Option<bool>,
    pub content: DetailWidgetContent,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct TextFieldWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "label")]
    pub label: Option<String>,
    #[serde(rename = "value")]
    pub value: Option<String>,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct PasswordFieldWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "label")]
    pub label: Option<String>,
    #[serde(rename = "value")]
    pub value: Option<String>,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct CheckboxWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "label")]
    pub label: Option<String>,
    #[serde(rename = "title")]
    pub title: Option<String>,
    #[serde(rename = "value")]
    pub value: Option<bool>,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct DatePickerWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "label")]
    pub label: Option<String>,
    #[serde(rename = "value")]
    pub value: Option<String>,
}
#[derive(Debug, Encode, Decode)]
pub struct SelectItemWidgetContent {
    pub text: Vec<String>,
}
impl<'de> Deserialize<'de> for SelectItemWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum SelectItemWidgetMembersOwned {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: String
            },
        }
        let mut members = Vec::<SelectItemWidgetMembersOwned>::deserialize(deserializer)?;

        let mut text = vec![];

        while let Some(member) = members.pop() {
            match member {
                SelectItemWidgetMembersOwned::Text { value } => {
                    text.insert(0, value);
                }
            }
        }

        Ok(SelectItemWidgetContent {
            text
        })
    }
}
impl Serialize for SelectItemWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum SelectItemWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_part")]
            Text {
                value: &'a String
            },
        }
        let mut members = Vec::<SelectItemWidgetMembersRef>::new();

        for value in &self.text {
            members.push(SelectItemWidgetMembersRef::Text { value });
        }

        Vec::<SelectItemWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct SelectItemWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "value")]
    pub value: String,
    pub content: SelectItemWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub enum SelectWidgetOrderedMembers {
    SelectItem(SelectItemWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct SelectWidgetContent {
    pub ordered_members: Vec<SelectWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for SelectWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum SelectWidgetMembersOwned {
            #[serde(rename = "gauntlet:select_item")]
            SelectItem(SelectItemWidget),
        }
        let mut members = Vec::<SelectWidgetMembersOwned>::deserialize(deserializer)?;

        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                SelectWidgetMembersOwned::SelectItem(widget) => {
                    ordered_members.insert(0, SelectWidgetOrderedMembers::SelectItem(widget));
                }
            }
        }

        Ok(SelectWidgetContent {
            ordered_members
        })
    }
}
impl Serialize for SelectWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum SelectWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:select_item")]
            SelectItem(&'a SelectItemWidget),
        }
        let mut members = Vec::<SelectWidgetMembersRef>::new();

        for member in &self.ordered_members {
            match member {
                SelectWidgetOrderedMembers::SelectItem(widget) => {
                    members.push(SelectWidgetMembersRef::SelectItem(widget))
                }
            }
        }

        Vec::<SelectWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct SelectWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "label")]
    pub label: Option<String>,
    #[serde(rename = "value")]
    pub value: Option<String>,
    pub content: SelectWidgetContent,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct SeparatorWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
}
#[derive(Debug, Encode, Decode)]
pub enum FormWidgetOrderedMembers {
    TextField(TextFieldWidget),
    PasswordField(PasswordFieldWidget),
    Checkbox(CheckboxWidget),
    DatePicker(DatePickerWidget),
    Select(SelectWidget),
    Separator(SeparatorWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct FormWidgetContent {
    pub actions: Option<ActionPanelWidget>,
    pub ordered_members: Vec<FormWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for FormWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum FormWidgetMembersOwned {
            #[serde(rename = "gauntlet:text_field")]
            TextField(TextFieldWidget),
            #[serde(rename = "gauntlet:password_field")]
            PasswordField(PasswordFieldWidget),
            #[serde(rename = "gauntlet:checkbox")]
            Checkbox(CheckboxWidget),
            #[serde(rename = "gauntlet:date_picker")]
            DatePicker(DatePickerWidget),
            #[serde(rename = "gauntlet:select")]
            Select(SelectWidget),
            #[serde(rename = "gauntlet:separator")]
            Separator(SeparatorWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(ActionPanelWidget),
        }
        let mut members = Vec::<FormWidgetMembersOwned>::deserialize(deserializer)?;

        let mut actions: Option<_> = None;
        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                FormWidgetMembersOwned::ActionPanel(widget) => {
                    if let Some(_) = actions {
                        return Err(Error::custom("Only one ActionPanel is allowed"))
                    }
                    actions = Some(widget);
                }
                FormWidgetMembersOwned::TextField(widget) => {
                    ordered_members.insert(0, FormWidgetOrderedMembers::TextField(widget));
                }
                FormWidgetMembersOwned::PasswordField(widget) => {
                    ordered_members.insert(0, FormWidgetOrderedMembers::PasswordField(widget));
                }
                FormWidgetMembersOwned::Checkbox(widget) => {
                    ordered_members.insert(0, FormWidgetOrderedMembers::Checkbox(widget));
                }
                FormWidgetMembersOwned::DatePicker(widget) => {
                    ordered_members.insert(0, FormWidgetOrderedMembers::DatePicker(widget));
                }
                FormWidgetMembersOwned::Select(widget) => {
                    ordered_members.insert(0, FormWidgetOrderedMembers::Select(widget));
                }
                FormWidgetMembersOwned::Separator(widget) => {
                    ordered_members.insert(0, FormWidgetOrderedMembers::Separator(widget));
                }
            }
        }

        Ok(FormWidgetContent {
            actions,
            ordered_members
        })
    }
}
impl Serialize for FormWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum FormWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:text_field")]
            TextField(&'a TextFieldWidget),
            #[serde(rename = "gauntlet:password_field")]
            PasswordField(&'a PasswordFieldWidget),
            #[serde(rename = "gauntlet:checkbox")]
            Checkbox(&'a CheckboxWidget),
            #[serde(rename = "gauntlet:date_picker")]
            DatePicker(&'a DatePickerWidget),
            #[serde(rename = "gauntlet:select")]
            Select(&'a SelectWidget),
            #[serde(rename = "gauntlet:separator")]
            Separator(&'a SeparatorWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(&'a ActionPanelWidget),
        }
        let mut members = Vec::<FormWidgetMembersRef>::new();

        if let Some(actions) = &self.actions {
            members.push(FormWidgetMembersRef::ActionPanel(actions))
        }
        for member in &self.ordered_members {
            match member {
                FormWidgetOrderedMembers::TextField(widget) => {
                    members.push(FormWidgetMembersRef::TextField(widget))
                }
                FormWidgetOrderedMembers::PasswordField(widget) => {
                    members.push(FormWidgetMembersRef::PasswordField(widget))
                }
                FormWidgetOrderedMembers::Checkbox(widget) => {
                    members.push(FormWidgetMembersRef::Checkbox(widget))
                }
                FormWidgetOrderedMembers::DatePicker(widget) => {
                    members.push(FormWidgetMembersRef::DatePicker(widget))
                }
                FormWidgetOrderedMembers::Select(widget) => {
                    members.push(FormWidgetMembersRef::Select(widget))
                }
                FormWidgetOrderedMembers::Separator(widget) => {
                    members.push(FormWidgetMembersRef::Separator(widget))
                }
            }
        }

        Vec::<FormWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct FormWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "isLoading")]
    pub is_loading: Option<bool>,
    pub content: FormWidgetContent,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct InlineSeparatorWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "icon")]
    pub icon: Option<Icons>,
}
#[derive(Debug, Encode, Decode)]
pub enum InlineWidgetOrderedMembers {
    Content(ContentWidget),
    InlineSeparator(InlineSeparatorWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct InlineWidgetContent {
    pub actions: Option<ActionPanelWidget>,
    pub ordered_members: Vec<InlineWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for InlineWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum InlineWidgetMembersOwned {
            #[serde(rename = "gauntlet:content")]
            Content(ContentWidget),
            #[serde(rename = "gauntlet:inline_separator")]
            InlineSeparator(InlineSeparatorWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(ActionPanelWidget),
        }
        let mut members = Vec::<InlineWidgetMembersOwned>::deserialize(deserializer)?;

        let mut actions: Option<_> = None;
        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                InlineWidgetMembersOwned::ActionPanel(widget) => {
                    if let Some(_) = actions {
                        return Err(Error::custom("Only one ActionPanel is allowed"))
                    }
                    actions = Some(widget);
                }
                InlineWidgetMembersOwned::Content(widget) => {
                    ordered_members.insert(0, InlineWidgetOrderedMembers::Content(widget));
                }
                InlineWidgetMembersOwned::InlineSeparator(widget) => {
                    ordered_members.insert(0, InlineWidgetOrderedMembers::InlineSeparator(widget));
                }
            }
        }

        Ok(InlineWidgetContent {
            actions,
            ordered_members
        })
    }
}
impl Serialize for InlineWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum InlineWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:content")]
            Content(&'a ContentWidget),
            #[serde(rename = "gauntlet:inline_separator")]
            InlineSeparator(&'a InlineSeparatorWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(&'a ActionPanelWidget),
        }
        let mut members = Vec::<InlineWidgetMembersRef>::new();

        if let Some(actions) = &self.actions {
            members.push(InlineWidgetMembersRef::ActionPanel(actions))
        }
        for member in &self.ordered_members {
            match member {
                InlineWidgetOrderedMembers::Content(widget) => {
                    members.push(InlineWidgetMembersRef::Content(widget))
                }
                InlineWidgetOrderedMembers::InlineSeparator(widget) => {
                    members.push(InlineWidgetMembersRef::InlineSeparator(widget))
                }
            }
        }

        Vec::<InlineWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct InlineWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    pub content: InlineWidgetContent,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct EmptyViewWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "title")]
    pub title: String,
    #[serde(rename = "description")]
    pub description: Option<String>,
    #[serde(rename = "image")]
    pub image: Option<ImageLike>,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct IconAccessoryWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "icon")]
    pub icon: ImageLike,
    #[serde(rename = "tooltip")]
    pub tooltip: Option<String>,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct TextAccessoryWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "text")]
    pub text: String,
    #[serde(rename = "icon")]
    pub icon: Option<ImageLike>,
    #[serde(rename = "tooltip")]
    pub tooltip: Option<String>,
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct SearchBarWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "value")]
    pub value: Option<String>,
    #[serde(rename = "placeholder")]
    pub placeholder: Option<String>,
}
#[derive(Debug, Encode, Decode)]
pub struct ListItemWidgetContent {
    pub accessories: Vec<ListItemAccessories>,
}
impl<'de> Deserialize<'de> for ListItemWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum ListItemWidgetMembersOwned {
            #[serde(rename = "gauntlet:accessory_text")]
            TextAccessory(TextAccessoryWidget),
            #[serde(rename = "gauntlet:accessory_icon")]
            IconAccessory(IconAccessoryWidget),
        }
        let mut members = Vec::<ListItemWidgetMembersOwned>::deserialize(deserializer)?;

        let mut accessories: Vec<_> = vec![];

        while let Some(member) = members.pop() {
            match member {
                ListItemWidgetMembersOwned::TextAccessory(widget) => {
                    accessories.push(ListItemAccessories::_0(widget));
                }
                ListItemWidgetMembersOwned::IconAccessory(widget) => {
                    accessories.push(ListItemAccessories::_1(widget));
                }
            }
        }

        Ok(ListItemWidgetContent {
            accessories,
        })
    }
}
impl Serialize for ListItemWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum ListItemWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:accessory_text")]
            TextAccessory(&'a TextAccessoryWidget),
            #[serde(rename = "gauntlet:accessory_icon")]
            IconAccessory(&'a IconAccessoryWidget),
        }
        let mut members = Vec::<ListItemWidgetMembersRef>::new();

        for item in &self.accessories {
            match item {
                ListItemAccessories::_0(widget) => {
                    members.push(ListItemWidgetMembersRef::TextAccessory(widget));
                }
                ListItemAccessories::_1(widget) => {
                    members.push(ListItemWidgetMembersRef::IconAccessory(widget));
                }
            }
        }

        Vec::<ListItemWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ListItemWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "id")]
    pub id: String,
    #[serde(rename = "title")]
    pub title: String,
    #[serde(rename = "subtitle")]
    pub subtitle: Option<String>,
    #[serde(rename = "icon")]
    pub icon: Option<ImageLike>,
    pub content: ListItemWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub enum ListItemAccessories {
    _0(TextAccessoryWidget),
    _1(IconAccessoryWidget),
}

#[derive(Debug, Encode, Decode)]
pub enum ListSectionWidgetOrderedMembers {
    ListItem(ListItemWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct ListSectionWidgetContent {
    pub ordered_members: Vec<ListSectionWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for ListSectionWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum ListSectionWidgetMembersOwned {
            #[serde(rename = "gauntlet:list_item")]
            ListItem(ListItemWidget),
        }
        let mut members = Vec::<ListSectionWidgetMembersOwned>::deserialize(deserializer)?;

        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                ListSectionWidgetMembersOwned::ListItem(widget) => {
                    ordered_members.insert(0, ListSectionWidgetOrderedMembers::ListItem(widget));
                }
            }
        }

        Ok(ListSectionWidgetContent {
            ordered_members
        })
    }
}
impl Serialize for ListSectionWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum ListSectionWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:list_item")]
            ListItem(&'a ListItemWidget),
        }
        let mut members = Vec::<ListSectionWidgetMembersRef>::new();

        for member in &self.ordered_members {
            match member {
                ListSectionWidgetOrderedMembers::ListItem(widget) => {
                    members.push(ListSectionWidgetMembersRef::ListItem(widget))
                }
            }
        }

        Vec::<ListSectionWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ListSectionWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "title")]
    pub title: String,
    #[serde(rename = "subtitle")]
    pub subtitle: Option<String>,
    pub content: ListSectionWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub enum ListWidgetOrderedMembers {
    ListItem(ListItemWidget),
    ListSection(ListSectionWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct ListWidgetContent {
    pub actions: Option<ActionPanelWidget>,
    pub search_bar: Option<SearchBarWidget>,
    pub empty_view: Option<EmptyViewWidget>,
    pub detail: Option<DetailWidget>,
    pub ordered_members: Vec<ListWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for ListWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum ListWidgetMembersOwned {
            #[serde(rename = "gauntlet:list_item")]
            ListItem(ListItemWidget),
            #[serde(rename = "gauntlet:list_section")]
            ListSection(ListSectionWidget),
            #[serde(rename = "gauntlet:search_bar")]
            SearchBar(SearchBarWidget),
            #[serde(rename = "gauntlet:empty_view")]
            EmptyView(EmptyViewWidget),
            #[serde(rename = "gauntlet:detail")]
            Detail(DetailWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(ActionPanelWidget),
        }
        let mut members = Vec::<ListWidgetMembersOwned>::deserialize(deserializer)?;

        let mut actions: Option<_> = None;
        let mut search_bar: Option<_> = None;
        let mut empty_view: Option<_> = None;
        let mut detail: Option<_> = None;
        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                ListWidgetMembersOwned::ActionPanel(widget) => {
                    if let Some(_) = actions {
                        return Err(Error::custom("Only one ActionPanel is allowed"))
                    }
                    actions = Some(widget);
                }
                ListWidgetMembersOwned::SearchBar(widget) => {
                    if let Some(_) = search_bar {
                        return Err(Error::custom("Only one SearchBar is allowed"))
                    }
                    search_bar = Some(widget);
                }
                ListWidgetMembersOwned::EmptyView(widget) => {
                    if let Some(_) = empty_view {
                        return Err(Error::custom("Only one EmptyView is allowed"))
                    }
                    empty_view = Some(widget);
                }
                ListWidgetMembersOwned::Detail(widget) => {
                    if let Some(_) = detail {
                        return Err(Error::custom("Only one Detail is allowed"))
                    }
                    detail = Some(widget);
                }
                ListWidgetMembersOwned::ListItem(widget) => {
                    ordered_members.insert(0, ListWidgetOrderedMembers::ListItem(widget));
                }
                ListWidgetMembersOwned::ListSection(widget) => {
                    ordered_members.insert(0, ListWidgetOrderedMembers::ListSection(widget));
                }
            }
        }

        Ok(ListWidgetContent {
            search_bar,
            empty_view,
            detail,
            actions,
            ordered_members
        })
    }
}
impl Serialize for ListWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum ListWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:list_item")]
            ListItem(&'a ListItemWidget),
            #[serde(rename = "gauntlet:list_section")]
            ListSection(&'a ListSectionWidget),
            #[serde(rename = "gauntlet:search_bar")]
            SearchBar(&'a SearchBarWidget),
            #[serde(rename = "gauntlet:empty_view")]
            EmptyView(&'a EmptyViewWidget),
            #[serde(rename = "gauntlet:detail")]
            Detail(&'a DetailWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(&'a ActionPanelWidget),
        }
        let mut members = Vec::<ListWidgetMembersRef>::new();

        if let Some(actions) = &self.actions {
            members.push(ListWidgetMembersRef::ActionPanel(actions))
        }
        if let Some(item) = &self.search_bar {
            members.push(ListWidgetMembersRef::SearchBar(item))
        }
        if let Some(item) = &self.empty_view {
            members.push(ListWidgetMembersRef::EmptyView(item))
        }
        if let Some(item) = &self.detail {
            members.push(ListWidgetMembersRef::Detail(item))
        }
        for member in &self.ordered_members {
            match member {
                ListWidgetOrderedMembers::ListItem(widget) => {
                    members.push(ListWidgetMembersRef::ListItem(widget))
                }
                ListWidgetOrderedMembers::ListSection(widget) => {
                    members.push(ListWidgetMembersRef::ListSection(widget))
                }
            }
        }

        Vec::<ListWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ListWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "isLoading")]
    pub is_loading: Option<bool>,
    pub content: ListWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub struct GridItemWidgetContent {
    pub accessory: Option<IconAccessoryWidget>,
    pub content: ContentWidget,
}
impl<'de> Deserialize<'de> for GridItemWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum GridItemWidgetMembersOwned {
            #[serde(rename = "gauntlet:content")]
            Content(ContentWidget),
            #[serde(rename = "gauntlet:accessory_icon")]
            IconAccessory(IconAccessoryWidget),
        }
        let mut members = Vec::<GridItemWidgetMembersOwned>::deserialize(deserializer)?;

        let mut accessory: Option<_> = None;
        let mut content: Option<_> = None;

        while let Some(member) = members.pop() {
            match member {
                GridItemWidgetMembersOwned::IconAccessory(widget) => {
                    if let Some(_) = accessory {
                        return Err(Error::custom("Only one IconAccessory is allowed"))
                    }
                    accessory = Some(widget);
                }
                GridItemWidgetMembersOwned::Content(widget) => {
                    if let Some(_) = content {
                        return Err(Error::custom("Only one Content is allowed"))
                    }
                    content = Some(widget);
                }
            }
        }

        Ok(GridItemWidgetContent {
            content: content.ok_or(Error::custom("Content is required"))?,
            accessory,
        })
    }
}
impl Serialize for GridItemWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum GridItemWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:content")]
            Content(&'a ContentWidget),
            #[serde(rename = "gauntlet:accessory_icon")]
            IconAccessory(&'a IconAccessoryWidget),
        }
        let mut members = Vec::<GridItemWidgetMembersRef>::new();

        if let Some(accessory) = &self.accessory {
            members.push(GridItemWidgetMembersRef::IconAccessory(accessory))
        }
        members.push(GridItemWidgetMembersRef::Content(&self.content));

        Vec::<GridItemWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct GridItemWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "id")]
    pub id: String,
    #[serde(rename = "title")]
    pub title: Option<String>,
    #[serde(rename = "subtitle")]
    pub subtitle: Option<String>,
    pub content: GridItemWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub enum GridSectionWidgetOrderedMembers {
    GridItem(GridItemWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct GridSectionWidgetContent {
    pub ordered_members: Vec<GridSectionWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for GridSectionWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum GridSectionWidgetMembersOwned {
            #[serde(rename = "gauntlet:grid_item")]
            GridItem(GridItemWidget),
        }
        let mut members = Vec::<GridSectionWidgetMembersOwned>::deserialize(deserializer)?;

        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                GridSectionWidgetMembersOwned::GridItem(widget) => {
                    ordered_members.insert(0, GridSectionWidgetOrderedMembers::GridItem(widget));
                }
            }
        }

        Ok(GridSectionWidgetContent {
            ordered_members
        })
    }
}
impl Serialize for GridSectionWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum GridSectionWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:grid_item")]
            GridItem(&'a GridItemWidget),
        }
        let mut members = Vec::<GridSectionWidgetMembersRef>::new();

        for member in &self.ordered_members {
            match member {
                GridSectionWidgetOrderedMembers::GridItem(widget) => {
                    members.push(GridSectionWidgetMembersRef::GridItem(widget))
                }
            }
        }

        Vec::<GridSectionWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct GridSectionWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "title")]
    pub title: String,
    #[serde(rename = "subtitle")]
    pub subtitle: Option<String>,
    #[serde(rename = "columns")]
    pub columns: Option<f64>,
    pub content: GridSectionWidgetContent,
}
#[derive(Debug, Encode, Decode)]
pub enum GridWidgetOrderedMembers {
    GridItem(GridItemWidget),
    GridSection(GridSectionWidget),
}
#[derive(Debug, Encode, Decode)]
pub struct GridWidgetContent {
    pub actions: Option<ActionPanelWidget>,
    pub search_bar: Option<SearchBarWidget>,
    pub empty_view: Option<EmptyViewWidget>,
    pub ordered_members: Vec<GridWidgetOrderedMembers>,
}
impl<'de> Deserialize<'de> for GridWidgetContent {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        #[derive(Debug, Deserialize)]
        #[serde(tag = "__type__")]
        enum GridWidgetMembersOwned {
            #[serde(rename = "gauntlet:grid_item")]
            GridItem(GridItemWidget),
            #[serde(rename = "gauntlet:grid_section")]
            GridSection(GridSectionWidget),
            #[serde(rename = "gauntlet:search_bar")]
            SearchBar(SearchBarWidget),
            #[serde(rename = "gauntlet:empty_view")]
            EmptyView(EmptyViewWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(ActionPanelWidget),
        }
        let mut members = Vec::<GridWidgetMembersOwned>::deserialize(deserializer)?;

        let mut actions: Option<_> = None;
        let mut search_bar: Option<_> = None;
        let mut empty_view: Option<_> = None;
        let mut ordered_members = vec![];

        while let Some(member) = members.pop() {
            match member {
                GridWidgetMembersOwned::ActionPanel(widget) => {
                    if let Some(_) = actions {
                        return Err(Error::custom("Only one ActionPanel is allowed"))
                    }
                    actions = Some(widget);
                }
                GridWidgetMembersOwned::SearchBar(widget) => {
                    if let Some(_) = search_bar {
                        return Err(Error::custom("Only one SearchBar is allowed"))
                    }
                    search_bar = Some(widget);
                }
                GridWidgetMembersOwned::EmptyView(widget) => {
                    if let Some(_) = empty_view {
                        return Err(Error::custom("Only one EmptyView is allowed"))
                    }
                    empty_view = Some(widget);
                }
                GridWidgetMembersOwned::GridItem(widget) => {
                    ordered_members.insert(0, GridWidgetOrderedMembers::GridItem(widget));
                }
                GridWidgetMembersOwned::GridSection(widget) => {
                    ordered_members.insert(0, GridWidgetOrderedMembers::GridSection(widget));
                }
            }
        }

        Ok(GridWidgetContent {
            search_bar,
            empty_view,
            actions,
            ordered_members
        })
    }
}
impl Serialize for GridWidgetContent {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer
    {
        #[derive(Debug, Serialize)]
        #[serde(tag = "__type__")]
        enum GridWidgetMembersRef<'a> {
            #[serde(rename = "gauntlet:grid_item")]
            GridItem(&'a GridItemWidget),
            #[serde(rename = "gauntlet:grid_section")]
            GridSection(&'a GridSectionWidget),
            #[serde(rename = "gauntlet:search_bar")]
            SearchBar(&'a SearchBarWidget),
            #[serde(rename = "gauntlet:empty_view")]
            EmptyView(&'a EmptyViewWidget),
            #[serde(rename = "gauntlet:action_panel")]
            ActionPanel(&'a ActionPanelWidget),
        }
        let mut members = Vec::<GridWidgetMembersRef>::new();

        if let Some(actions) = &self.actions {
            members.push(GridWidgetMembersRef::ActionPanel(actions))
        }
        if let Some(item) = &self.search_bar {
            members.push(GridWidgetMembersRef::SearchBar(item))
        }
        if let Some(item) = &self.empty_view {
            members.push(GridWidgetMembersRef::EmptyView(item))
        }
        for member in &self.ordered_members {
            match member {
                GridWidgetOrderedMembers::GridItem(widget) => {
                    members.push(GridWidgetMembersRef::GridItem(widget))
                }
                GridWidgetOrderedMembers::GridSection(widget) => {
                    members.push(GridWidgetMembersRef::GridSection(widget))
                }
            }
        }

        Vec::<GridWidgetMembersRef>::serialize(&members, serializer)
    }
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct GridWidget {
    #[serde(rename = "__id__")]
    pub __id__: UiWidgetId,
    #[serde(rename = "isLoading")]
    pub is_loading: Option<bool>,
    #[serde(rename = "columns")]
    pub columns: Option<f64>,
    pub content: GridWidgetContent,
}
#[derive(Debug, Clone, Serialize, Deserialize, Encode, Decode)]
pub enum Icons {
    #[serde(rename = "PersonAdd")]
    PersonAdd,
    #[serde(rename = "Airplane")]
    Airplane,
    #[serde(rename = "Alarm")]
    Alarm,
    #[serde(rename = "AlignCentre")]
    AlignCentre,
    #[serde(rename = "AlignLeft")]
    AlignLeft,
    #[serde(rename = "AlignRight")]
    AlignRight,
    #[serde(rename = "ArrowClockwise")]
    ArrowClockwise,
    #[serde(rename = "ArrowCounterClockwise")]
    ArrowCounterClockwise,
    #[serde(rename = "ArrowDown")]
    ArrowDown,
    #[serde(rename = "ArrowLeft")]
    ArrowLeft,
    #[serde(rename = "ArrowRight")]
    ArrowRight,
    #[serde(rename = "ArrowUp")]
    ArrowUp,
    #[serde(rename = "ArrowLeftRight")]
    ArrowLeftRight,
    #[serde(rename = "ArrowsContract")]
    ArrowsContract,
    #[serde(rename = "ArrowsExpand")]
    ArrowsExpand,
    #[serde(rename = "AtSymbol")]
    AtSymbol,
    #[serde(rename = "Cash")]
    Cash,
    #[serde(rename = "Battery")]
    Battery,
    #[serde(rename = "BatteryCharging")]
    BatteryCharging,
    #[serde(rename = "Bell")]
    Bell,
    #[serde(rename = "BellDisabled")]
    BellDisabled,
    #[serde(rename = "Document")]
    Document,
    #[serde(rename = "DocumentAdd")]
    DocumentAdd,
    #[serde(rename = "DocumentDelete")]
    DocumentDelete,
    #[serde(rename = "Bluetooth")]
    Bluetooth,
    #[serde(rename = "Bold")]
    Bold,
    #[serde(rename = "Book")]
    Book,
    #[serde(rename = "Bookmark")]
    Bookmark,
    #[serde(rename = "Box")]
    Box,
    #[serde(rename = "Bug")]
    Bug,
    #[serde(rename = "Building")]
    Building,
    #[serde(rename = "BulletPoints")]
    BulletPoints,
    #[serde(rename = "Calculator")]
    Calculator,
    #[serde(rename = "Calendar")]
    Calendar,
    #[serde(rename = "Camera")]
    Camera,
    #[serde(rename = "Car")]
    Car,
    #[serde(rename = "Cart")]
    Cart,
    #[serde(rename = "Checkmark")]
    Checkmark,
    #[serde(rename = "ChevronDown")]
    ChevronDown,
    #[serde(rename = "ChevronLeft")]
    ChevronLeft,
    #[serde(rename = "ChevronRight")]
    ChevronRight,
    #[serde(rename = "ChevronUp")]
    ChevronUp,
    #[serde(rename = "ChevronExpand")]
    ChevronExpand,
    #[serde(rename = "Circle")]
    Circle,
    #[serde(rename = "Clipboard")]
    Clipboard,
    #[serde(rename = "Clock")]
    Clock,
    #[serde(rename = "Cloud")]
    Cloud,
    #[serde(rename = "CloudLightning")]
    CloudLightning,
    #[serde(rename = "CloudRain")]
    CloudRain,
    #[serde(rename = "CloudSnow")]
    CloudSnow,
    #[serde(rename = "CloudSun")]
    CloudSun,
    #[serde(rename = "Code")]
    Code,
    #[serde(rename = "Gear")]
    Gear,
    #[serde(rename = "Coin")]
    Coin,
    #[serde(rename = "Command")]
    Command,
    #[serde(rename = "Compass")]
    Compass,
    #[serde(rename = "CreditCard")]
    CreditCard,
    #[serde(rename = "Crop")]
    Crop,
    #[serde(rename = "Dot")]
    Dot,
    #[serde(rename = "Download")]
    Download,
    #[serde(rename = "Eject")]
    Eject,
    #[serde(rename = "ThreeDots")]
    ThreeDots,
    #[serde(rename = "Envelope")]
    Envelope,
    #[serde(rename = "Eraser")]
    Eraser,
    #[serde(rename = "ExclamationMark")]
    ExclamationMark,
    #[serde(rename = "Eye")]
    Eye,
    #[serde(rename = "EyeDisabled")]
    EyeDisabled,
    #[serde(rename = "EyeDropper")]
    EyeDropper,
    #[serde(rename = "Female")]
    Female,
    #[serde(rename = "Film")]
    Film,
    #[serde(rename = "Filter")]
    Filter,
    #[serde(rename = "Fingerprint")]
    Fingerprint,
    #[serde(rename = "Flag")]
    Flag,
    #[serde(rename = "Folder")]
    Folder,
    #[serde(rename = "FolderAdd")]
    FolderAdd,
    #[serde(rename = "FolderDelete")]
    FolderDelete,
    #[serde(rename = "Forward")]
    Forward,
    #[serde(rename = "GameController")]
    GameController,
    #[serde(rename = "Virus")]
    Virus,
    #[serde(rename = "Gift")]
    Gift,
    #[serde(rename = "Glasses")]
    Glasses,
    #[serde(rename = "Globe")]
    Globe,
    #[serde(rename = "Hammer")]
    Hammer,
    #[serde(rename = "HardDrive")]
    HardDrive,
    #[serde(rename = "Headphones")]
    Headphones,
    #[serde(rename = "Heart")]
    Heart,
    #[serde(rename = "Heartbeat")]
    Heartbeat,
    #[serde(rename = "Hourglass")]
    Hourglass,
    #[serde(rename = "House")]
    House,
    #[serde(rename = "Image")]
    Image,
    #[serde(rename = "Info")]
    Info,
    #[serde(rename = "Italics")]
    Italics,
    #[serde(rename = "Key")]
    Key,
    #[serde(rename = "Keyboard")]
    Keyboard,
    #[serde(rename = "Layers")]
    Layers,
    #[serde(rename = "LightBulb")]
    LightBulb,
    #[serde(rename = "LightBulbDisabled")]
    LightBulbDisabled,
    #[serde(rename = "Link")]
    Link,
    #[serde(rename = "List")]
    List,
    #[serde(rename = "Lock")]
    Lock,
    #[serde(rename = "LockUnlocked")]
    LockUnlocked,
    #[serde(rename = "Male")]
    Male,
    #[serde(rename = "Map")]
    Map,
    #[serde(rename = "Maximize")]
    Maximize,
    #[serde(rename = "Megaphone")]
    Megaphone,
    #[serde(rename = "MemoryModule")]
    MemoryModule,
    #[serde(rename = "MemoryStick")]
    MemoryStick,
    #[serde(rename = "Message")]
    Message,
    #[serde(rename = "Microphone")]
    Microphone,
    #[serde(rename = "MicrophoneDisabled")]
    MicrophoneDisabled,
    #[serde(rename = "Minimize")]
    Minimize,
    #[serde(rename = "Minus")]
    Minus,
    #[serde(rename = "Mobile")]
    Mobile,
    #[serde(rename = "Moon")]
    Moon,
    #[serde(rename = "Mouse")]
    Mouse,
    #[serde(rename = "Multiply")]
    Multiply,
    #[serde(rename = "Music")]
    Music,
    #[serde(rename = "Network")]
    Network,
    #[serde(rename = "Paperclip")]
    Paperclip,
    #[serde(rename = "Paragraph")]
    Paragraph,
    #[serde(rename = "Pause")]
    Pause,
    #[serde(rename = "Pencil")]
    Pencil,
    #[serde(rename = "Person")]
    Person,
    #[serde(rename = "Phone")]
    Phone,
    #[serde(rename = "PieChart")]
    PieChart,
    #[serde(rename = "Capsule")]
    Capsule,
    #[serde(rename = "Play")]
    Play,
    #[serde(rename = "Plug")]
    Plug,
    #[serde(rename = "Plus")]
    Plus,
    #[serde(rename = "Power")]
    Power,
    #[serde(rename = "Printer")]
    Printer,
    #[serde(rename = "QuestionMark")]
    QuestionMark,
    #[serde(rename = "Quotes")]
    Quotes,
    #[serde(rename = "Receipt")]
    Receipt,
    #[serde(rename = "PersonRemove")]
    PersonRemove,
    #[serde(rename = "Repeat")]
    Repeat,
    #[serde(rename = "Reply")]
    Reply,
    #[serde(rename = "Rewind")]
    Rewind,
    #[serde(rename = "Rocket")]
    Rocket,
    #[serde(rename = "Shield")]
    Shield,
    #[serde(rename = "Shuffle")]
    Shuffle,
    #[serde(rename = "Snippets")]
    Snippets,
    #[serde(rename = "Snowflake")]
    Snowflake,
    #[serde(rename = "Star")]
    Star,
    #[serde(rename = "Stop")]
    Stop,
    #[serde(rename = "Stopwatch")]
    Stopwatch,
    #[serde(rename = "StrikeThrough")]
    StrikeThrough,
    #[serde(rename = "Sun")]
    Sun,
    #[serde(rename = "Scissors")]
    Scissors,
    #[serde(rename = "Tag")]
    Tag,
    #[serde(rename = "Thermometer")]
    Thermometer,
    #[serde(rename = "Terminal")]
    Terminal,
    #[serde(rename = "Text")]
    Text,
    #[serde(rename = "TextCursor")]
    TextCursor,
    #[serde(rename = "Trash")]
    Trash,
    #[serde(rename = "Tree")]
    Tree,
    #[serde(rename = "Trophy")]
    Trophy,
    #[serde(rename = "People")]
    People,
    #[serde(rename = "Umbrella")]
    Umbrella,
    #[serde(rename = "Underline")]
    Underline,
    #[serde(rename = "Upload")]
    Upload,
    #[serde(rename = "Wallet")]
    Wallet,
    #[serde(rename = "Wand")]
    Wand,
    #[serde(rename = "Wifi")]
    Wifi,
    #[serde(rename = "WifiDisabled")]
    WifiDisabled,
    #[serde(rename = "Window")]
    Window,
    #[serde(rename = "Tools")]
    Tools,
    #[serde(rename = "Watch")]
    Watch,
    #[serde(rename = "XMark")]
    XMark,
    #[serde(rename = "Indent")]
    Indent,
    #[serde(rename = "Unindent")]
    Unindent,
}

#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ImageSourceUrl {
    pub url: String,
}

#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct ImageSourceAsset {
    pub asset: String,
}

#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
#[serde(untagged)]
pub enum ImageSource {
    ImageSourceUrl(ImageSourceUrl),
    ImageSourceAsset(ImageSourceAsset),
}

#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
#[serde(untagged)]
pub enum ImageLike {
    ImageSource(ImageSource),
    Icons(Icons),
}

#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
#[serde(tag = "__type__")]
pub enum RootWidgetMembers {
    #[serde(rename = "gauntlet:detail")]
    Detail(DetailWidget),
    #[serde(rename = "gauntlet:form")]
    Form(FormWidget),
    #[serde(rename = "gauntlet:inline")]
    Inline(InlineWidget),
    #[serde(rename = "gauntlet:list")]
    List(ListWidget),
    #[serde(rename = "gauntlet:grid")]
    Grid(GridWidget),
}
#[derive(Debug, Serialize, Deserialize, Encode, Decode)]
pub struct RootWidget {
    #[serde(default, deserialize_with = "array_to_option", serialize_with = "option_to_array")]
    pub content: Option<RootWidgetMembers>
}
